Imports System
Imports System.Text
Imports System.IO
Imports System.Net
Imports System.Net.Sockets

Public Class cTelnetClient

    Private Const AUTH_LOGIN As Long = 100
    Private Const AUTH_PASSWORD As Long = 101
    Private Const AUTH_WAIT As Long = 102
    Private Const AUTH_DONE As Long = 103

    Private Const AUTH_STATUS_SUCCESS As Long = 103
    Private Const AUTH_STATUS_FAIL As Long = 104
    Private Const AUTH_STATUS_WAIT As Long = 105

    Private m_objSocket As Socket = Nothing

    Private m_sLoginTelnet As String
    Private m_sPasswordTelnet As String
    Private m_sRecvBuffer As String
    Private m_nAuthStep As Integer
    Private m_nAuthStatus As Integer

    Private Const RECV_BUFFER_SIZE As Long = 512
    Private m_sAsyncBuffer(RECV_BUFFER_SIZE) As Byte

    Private Sub OnReceive(ByVal ar As IAsyncResult)

        SyncLock (Me)

            If m_objSocket Is Nothing Then
                Exit Sub
            End If

            Try
                Dim nRead As Integer = m_objSocket.EndReceive(ar)

                If nRead = 0 Then
                    OnDisconnect()
                    Exit Sub
                End If

                m_sRecvBuffer = m_sRecvBuffer + Encoding.ASCII.GetString(m_sAsyncBuffer, 0, nRead)
                m_sRecvBuffer = m_sRecvBuffer.Replace("VaxTele:>", "")

                ReceiveData()

                If m_nAuthStatus <> AUTH_STATUS_FAIL Then
                    m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, New AsyncCallback(AddressOf OnReceive), 0)
                End If

            Catch e As SocketException
                OnDisconnect()
            End Try


        End SyncLock


    End Sub

    Private Sub ReceiveData()

        While True

            If m_nAuthStep <> AUTH_DONE Then
                OnAuthrization(m_sRecvBuffer)
                Exit While
            End If

            Dim sRequest As String = FindRequest()
            If sRequest.Length = 0 Then
                Exit While
            End If

            Dim sCommand As String = ""
            Dim objParamList As New ArrayList

            If ParseRequest(sRequest, sCommand, objParamList) = False Then
                Exit While
            End If

            OnRequest(sCommand, objParamList)

        End While

    End Sub

    Private Sub OnRequest(ByVal sCommand As String, ByRef objParamList As ArrayList)

        If String.Compare(sCommand, "BYE", True) = 0 Then
            OnDisconnect()
            Exit Sub
        End If

        MainForm.m_objMainForm.InvokeTelnetRecv(sCommand, objParamList)

    End Sub

    Public Function Connect(ByVal sToIP As String, ByVal nToPort As Integer, ByVal sLogin As String, ByVal sPassword As String) As Boolean

        CloseSocket()
        m_objSocket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)

        m_objSocket.BeginConnect(sToIP, nToPort, Nothing, 0)

        Dim nCount As Integer
        For nCount = 0 To 30
            Threading.Thread.Sleep(100)
            If m_objSocket.Connected = True Then
                Exit For
            End If
        Next

        If m_objSocket.Connected = False Then
            Connect = False
            MessageBox.Show("Fail to connect to server.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Function
        End If

        m_sLoginTelnet = sLogin
        m_sPasswordTelnet = sPassword

        m_nAuthStep = AUTH_LOGIN
        m_nAuthStatus = AUTH_STATUS_WAIT
        m_sRecvBuffer = ""

        m_objSocket.BeginReceive(m_sAsyncBuffer, 0, RECV_BUFFER_SIZE, SocketFlags.None, New AsyncCallback(AddressOf OnReceive), 0)

        For nCount = 0 To 40
            Threading.Thread.Sleep(100)
            If m_nAuthStatus <> AUTH_STATUS_WAIT Then
                Exit For
            End If
        Next

        If m_nAuthStatus <> AUTH_STATUS_SUCCESS Then
            Connect = False
            MessageBox.Show("Fail to connect to server.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Exit Function
        End If

        Connect = True
        MessageBox.Show("Connected successfully.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

    End Function

    Public Function FindRequest() As String

        Dim nStart As Integer = m_sRecvBuffer.IndexOf(vbCrLf)

        If nStart = -1 Then
            FindRequest = ""
            Exit Function
        End If

        FindRequest = m_sRecvBuffer.Substring(0, nStart)
        m_sRecvBuffer = m_sRecvBuffer.Remove(0, nStart + 2)

    End Function

    Public Sub OnAuthrization(ByVal sData As String)

        If m_nAuthStep = AUTH_LOGIN Then

            If sData.IndexOf("Login:") = -1 Then
                Exit Sub
            End If

            m_nAuthStep = AUTH_PASSWORD
            SendCommand(m_sLoginTelnet, Nothing)

            m_sRecvBuffer = ""

        ElseIf m_nAuthStep = AUTH_PASSWORD Then

            If sData.IndexOf("Password:") = -1 Then
                Exit Sub
            End If

            m_nAuthStep = AUTH_WAIT
            SendCommand(m_sPasswordTelnet, Nothing)

            m_sRecvBuffer = ""

        ElseIf m_nAuthStep = AUTH_WAIT And sData.Length <> 0 Then

            m_sRecvBuffer = ""

            If sData.IndexOf("Success") = -1 Then

                m_nAuthStep = AUTH_LOGIN
                m_nAuthStatus = AUTH_STATUS_FAIL

                Exit Sub
            End If

            m_nAuthStep = AUTH_DONE
            m_nAuthStatus = AUTH_STATUS_SUCCESS

        End If

    End Sub

    Private Function ParseRequest(ByVal sRequest As String, ByRef sCommand As String, ByRef objParamList As ArrayList) As Boolean

        Dim nEnd As Integer = sRequest.IndexOf(" ")

        If nEnd = -1 Then
            sCommand = sRequest
            ParseRequest = True
            Exit Function
        End If

        sCommand = sRequest.Substring(0, nEnd)
        sRequest = sRequest.Remove(0, nEnd + 1)

        If String.Compare(sCommand, "MSG", True) = 0 Or String.Compare(sCommand, "StatusServerSIP", True) = 0 Or String.Compare(sCommand, "ErrorLog", True) = 0 Or String.Compare(sCommand, "StatusLog", True) = 0 Then
            objParamList.Add(sRequest)
            ParseRequest = True
            Exit Function
        End If

        While True
            nEnd = sRequest.IndexOf(" ")

            If nEnd = -1 Then
                objParamList.Add(sRequest)
                Exit While
            End If

            objParamList.Add(sRequest.Substring(0, nEnd))
            sRequest = sRequest.Remove(0, nEnd + 1)

        End While

        ParseRequest = True

    End Function

    Public Sub SendCommand(ByVal sCommand As String, ByRef objParamList As ArrayList)

        If m_objSocket Is Nothing Then
            Exit Sub
        End If

        Dim sToData As String = sCommand

        If Not objParamList Is Nothing Then

            Dim nCount As Integer
            For nCount = 0 To objParamList.Count - 1
                sToData = sToData + " " + objParamList.Item(nCount)
            Next

        End If

        sToData = sToData + vbCrLf
        m_objSocket.Send(Encoding.UTF8.GetBytes(sToData), 0, sToData.Length, SocketFlags.None)

    End Sub

    Public Sub CloseSocket()

        If Not m_objSocket Is Nothing Then

            SyncLock (Me)

                m_objSocket.Close()
                m_objSocket = Nothing

            End SyncLock

        End If

        m_nAuthStatus = AUTH_STATUS_FAIL

    End Sub

    Public Sub OnDisconnect()

        CloseSocket()
        MessageBox.Show("Connection to host lost.", "Socket", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

    End Sub


End Class
